@@ -1452,7 +1452,7 @@ static void print_compressed_tables(Block *b)/*{{{*/
 }
 /*}}}*/
 /* ================================================================= */
-void yyerror (char *s)/*{{{*/
+void yyerror (const char *s)/*{{{*/
 {
   extern int lineno;
   fprintf(stderr, "%s at line %d\n", s, lineno);
