@@ -48,14 +48,23 @@ static char *database_path = NULL;
 static enum folder_type output_folder_type = FT_MAILDIR;
 static int skip_integrity_checks = 0;
 
-static int file_exists(char *name)/*{{{*/
+enum filetype {
+  M_NONE, M_FILE, M_DIR, M_OTHER
+};
+
+static enum filetype classify_file(char *name)/*{{{*/
 {
   struct stat sb;
-
   if (stat(name, &sb) < 0) {
-    return 0;
+    return M_NONE;
+  }
+  if (S_ISREG(sb.st_mode)) {
+    return M_FILE;
+  } else if (S_ISDIR(sb.st_mode)) {
+    return M_DIR;
+  } else {
+    return M_OTHER;
   }
-  return 1;
 }
 /*}}}*/
 /*{{{ member of*/
@@ -77,13 +86,13 @@ int member_of (const char *complete_mfolder,
   split_on_colons(folders, &n_raw_paths, &raw_paths);
   switch (ft) {
     case FT_MAILDIR:
-      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_maildir, omit_globs);
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, &maildir_traverse_methods, omit_globs);
       break;
     case FT_MH:
-      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_mh, omit_globs);
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, &mh_traverse_methods, omit_globs);
       break;
     case FT_MBOX:
-      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_file, omit_globs);
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, &mbox_traverse_methods, omit_globs);
       break;
     case FT_RAW:			/* cannot happen but to keep compiler happy */
       break;    
@@ -319,7 +328,7 @@ static void emit_int(int x)/*{{{*/
   return;
 }
 /*}}}*/
-volatile void out_of_mem(char *file, int line, size_t size)/*{{{*/
+void out_of_mem(char *file, int line, size_t size)/*{{{*/
 {
   /* Hairy coding ahead - can't use any [s]printf, itoa etc because
    * those might try to use the heap! */
@@ -353,6 +362,13 @@ static void print_copyright(void)/*{{{*/
           PROGRAM_VERSION);
 }
 /*}}}*/
+static void print_version(void)/*{{{*/
+{
+  fprintf(stdout,
+          "mairix %s\n",
+          PROGRAM_VERSION);
+}
+/*}}}*/
 static void usage(void)/*{{{*/
 {
   print_copyright();
@@ -363,6 +379,7 @@ static void usage(void)/*{{{*/
          "mairix [-f <rcfile>] -d                        : Dump database to stdout\n"
          "-h           : show this help\n"
          "-f <rcfile>  : use alternative rc file (default ~/.mairixrc)\n"
+         "-V           : show version\n"
          "-v           : be verbose\n"
          "-p           : purge messages that no longer exist\n"
          "-a           : add new matches to match folder (default : clear it first)\n"
@@ -370,7 +387,7 @@ static void usage(void)/*{{{*/
          "-o <mfolder> : override setting of mfolder from mairixrc file\n"
          "-r           : force raw output regardless of mformat setting in mairixrc file\n"
          "expr_i       : search expression (all expr's AND'ed together):\n"
-         "    word          : match word in whole message\n"
+         "    word          : match word in message body and major headers\n"
          "    t:word        : match word in To: header\n"
          "    c:word        : match word in Cc: header\n"
          "    f:word        : match word in From: header\n"
@@ -422,7 +439,7 @@ static void usage(void)/*{{{*/
 int main (int argc, char **argv)/*{{{*/
 {
   struct msgpath_array *msgs;
-  struct database *db;
+  struct database *db = NULL;
 
   char *arg_rc_file_path = NULL;
   char *arg_mfolder = NULL;
@@ -472,6 +489,9 @@ int main (int argc, char **argv)/*{{{*/
       do_forced_unlock = 1;
     } else if (!strcmp(*argv, "-v") || !strcmp(*argv, "--verbose")) {
       verbose = 1;
+    } else if (!strcmp(*argv, "-V") || !strcmp(*argv, "--version")) {
+      print_version();
+      exit(0);
     } else if (!strcmp(*argv, "-h") ||
                !strcmp(*argv, "--help")) {
       do_help = 1;
@@ -568,6 +588,7 @@ int main (int argc, char **argv)/*{{{*/
   } else if (do_search) {
     int len;
     char *complete_mfolder;
+    enum filetype ftype;
 
     if (!mfolder) {
       if (output_folder_type != FT_RAW) {
@@ -602,9 +623,16 @@ int main (int argc, char **argv)/*{{{*/
       unlock_and_exit(3);
     }
 
+    ftype = classify_file(database_path);
+    if (ftype != M_FILE) {
+      fprintf(stderr, "No database file '%s' is present.\nYou need to do an indexing run first.\n",
+          database_path);
+      unlock_and_exit(3);
+    }
     result = search_top(do_threads, do_augment, database_path, complete_mfolder, argv, output_folder_type, verbose);
     
   } else {
+    enum filetype ftype;
 
     if (!maildir_folders && !mh_folders && !mboxen) {
       fprintf(stderr, "No [mh_]folders/mboxen/MAIRIX_[MH_]FOLDERS set\n");
@@ -627,13 +655,17 @@ int main (int argc, char **argv)/*{{{*/
     }
 
     /* Try to open existing database */
-    if (file_exists(database_path)) {
+    ftype = classify_file(database_path);
+    if (ftype == M_FILE) {
       if (verbose) printf("Reading existing database...\n");
       db = new_database_from_file(database_path, do_integrity_checks);
       if (verbose) printf("Loaded %d existing messages\n", db->n_msgs);
-    } else {
+    } else if (ftype == M_NONE) {
       if (verbose) printf("Starting new database\n");
       db = new_database();
+    } else {
+      fprintf(stderr, "database path %s is not a file; you can't put the database there\n", database_path);
+      unlock_and_exit(2);
     }
 
     build_mbox_lists(db, folder_base, mboxen, omit_globs);
