@@ -101,11 +101,11 @@ static void append_new_mboxen_to_db(struct database *db, struct extant_mbox *ext
   db->n_mboxen = n_reqd;
 }
 /*}}}*/
-void compute_checksum(const unsigned char *data, size_t len, checksum_t *csum)/*{{{*/
+void compute_checksum(const char *data, size_t len, checksum_t *csum)/*{{{*/
 {
   MD5_CTX md5;
   MD5Init(&md5);
-  MD5Update(&md5, data, len);
+  MD5Update(&md5, (unsigned char *) data, len);
   MD5Final(&md5);
   memcpy(csum, md5.digest, sizeof(md5.digest));
   return;
@@ -132,7 +132,7 @@ static int message_is_intact(struct mbox *mb, int idx, char *va, size_t len)/*{{
   return 0;
 }
 /*}}}*/
-static int find_number_intact(struct mbox *mb, unsigned char *va, size_t len)/*{{{*/
+static int find_number_intact(struct mbox *mb, char *va, size_t len)/*{{{*/
 {
   /* Pick up the common obvious case first - where new messages have been appended to the
      end of the mbox */
@@ -199,9 +199,9 @@ static signed char fromcheck_table[256];
 
 #undef DEBUG_DFA
 
-static int looks_like_from_separator(off_t n, unsigned char *va, size_t len, int verbose)/*{{{*/
+static int looks_like_from_separator(off_t n, char *va, size_t len, int verbose)/*{{{*/
 {
-  unsigned char p;
+  char p;
   int current_state = 0;
   int result = 0;
 
@@ -210,9 +210,9 @@ static int looks_like_from_separator(off_t n, unsigned char *va, size_t len, int
   while (n < len) {
     p = va[n++];
     if (verbose) {
-      printf("current_state=%d, p=%02x (%1c) ", current_state, (int)p, ((p>=32)&&(p<=126))?p:'.');
+      printf("current_state=%d, p=%02x (%1c) ", current_state, (int)(unsigned char)p, ((p>=32)&&(p<=126))?p:'.');
     }
-    current_state = fromcheck_next_state(current_state, (int)fromcheck_table[(int)p]);
+    current_state = fromcheck_next_state(current_state, (int)fromcheck_table[(int)(unsigned char)p]);
     if (verbose) {
       printf("next_state=%d\n", current_state);
     }
@@ -255,7 +255,7 @@ static void init_fromcheck_table()/*{{{*/
   fromcheck_table['='] = FS_OTHEREMAIL;
 }
 /*}}}*/
-static off_t find_next_from(off_t n, unsigned char *va, size_t len)/*{{{*/
+static off_t find_next_from(off_t n, char *va, size_t len)/*{{{*/
 {
   unsigned char c;
   unsigned long hit;
@@ -300,7 +300,7 @@ scan_again:
   return -1;
 }
 /*}}}*/
-static off_t start_of_next_line(off_t n, unsigned char *va, size_t len)/*{{{*/
+static off_t start_of_next_line(off_t n, char *va, size_t len)/*{{{*/
 {
   unsigned char c;
   /* We are always starting from 'From ' so we can advance before testing */
@@ -318,7 +318,7 @@ static off_t start_of_next_line(off_t n, unsigned char *va, size_t len)/*{{{*/
 /*}}}*/
 
 
-static struct message_list *build_new_message_list(struct mbox *mb, unsigned char *va, size_t len, int *n_messages)/*{{{*/
+static struct message_list *build_new_message_list(struct mbox *mb, char *va, size_t len, int *n_messages)/*{{{*/
 {
   struct message_list *result, *here, *next;
   off_t start_from, start_pos, end_from;
@@ -393,7 +393,7 @@ done:
 
 }
 /*}}}*/
-static void append_new_messages(struct mbox *mb, struct message_list *list, int n, unsigned char *va)/*{{{*/
+static void append_new_messages(struct mbox *mb, struct message_list *list, int n)/*{{{*/
 {
   int j;
   struct message_list *here, *next;
@@ -417,7 +417,7 @@ static void append_new_messages(struct mbox *mb, struct message_list *list, int
   mb->n_msgs = N;
 }
 /*}}}*/
-static void rescan_mbox(struct mbox *mb, unsigned char *va, size_t len)/*{{{*/
+static void rescan_mbox(struct mbox *mb, char *va, size_t len)/*{{{*/
 {
   /* We get here if it's determined that
    * 1. the mbox file still exists
@@ -431,7 +431,7 @@ static void rescan_mbox(struct mbox *mb, unsigned char *va, size_t len)/*{{{*/
   /* Find the last message in the box that appears to be intact. */
   mb->n_old_msgs_valid = find_number_intact(mb, va, len);
   new_messages = build_new_message_list(mb, va, len, &n_new_messages);
-  append_new_messages(mb, new_messages, n_new_messages, va);
+  append_new_messages(mb, new_messages, n_new_messages);
 }
 /*}}}*/
 static void deaden_mbox(struct mbox *mb)/*{{{*/
@@ -521,11 +521,12 @@ static char *find_last_slash(char *in)/*{{{*/
   return result;
 }
 /*}}}*/
-static void append_shallow(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
-                  int (*filter)(const char *, struct stat *),
+static int append_shallow(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
+                  const struct traverse_methods *methods,
                   struct globber_array *omit_globs)
 {
-  if ((*filter)(path, sb)) {
+  int result = 0;
+  if ((methods->filter)(path, sb)) {
     if (!is_globber_array_match(omit_globs, path + base_len)) {
       struct string_list *nn = new(struct string_list);
       nn->data = new_string(path);
@@ -533,12 +534,14 @@ static void append_shallow(char *path, int base_len, struct stat *sb, struct str
       nn->prev = list->prev;
       list->prev->next = nn;
       list->prev = nn;
+      result = 1;
     }
   }
+  return result;
 }
 /*}}}*/
-static void append_deep(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
-                        int (*filter)(const char *, struct stat *), 
+static int append_deep(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
+                        const struct traverse_methods *methods,
                         struct globber_array *omit_globs)
 {
   /* path is dir : read its contents, call append_shallow or self accordingly. */
@@ -547,14 +550,18 @@ static void append_deep(char *path, int base_len, struct stat *sb, struct string
   char *xpath;
   DIR *d;
   struct dirent *de;
+  int appended_any = 0;
+  int this_file_matched;
 
-  append_shallow(path, base_len, sb, list, filter, omit_globs);
+  this_file_matched = append_shallow(path, base_len, sb, list, methods, omit_globs);
+  appended_any |= this_file_matched;
   
   if (S_ISDIR(sb->st_mode)) {
     xpath = new_array(char, strlen(path) + 2 + NAME_MAX);
     d = opendir(path);
     if (d) {
       while ((de = readdir(d))) {
+        enum traverse_check status;
         if (!strcmp(de->d_name, ".")) continue;
         if (!strcmp(de->d_name, "..")) continue;
         strcpy(xpath, path);
@@ -564,25 +571,46 @@ static void append_deep(char *path, int base_len, struct stat *sb, struct string
           /* Filter out omissions at this point, e.g. to avoid wasting time on
            * a recursive expansion of a tree that's going to get pruned in at
            * the deepest level anyway. */
-          if (stat(xpath, &sb2) >= 0) {
-            if (S_ISREG(sb2.st_mode)) {
-              append_shallow(xpath, base_len, &sb2, list, filter, omit_globs);
-            } else if (S_ISDIR(sb2.st_mode)) {
-              append_deep(xpath, base_len, &sb2, list, filter, omit_globs);
-            }
+          status = (methods->scrutinize)(this_file_matched, de->d_name);
+#if 0
+          /* debugging */
+          fprintf(stderr, "scrutinize for %s in %s returned %s\n",
+              de->d_name,
+              path,
+              (status == TRAV_FINISH) ? "FINISH" :
+              (status == TRAV_IGNORE) ? "IGNORE" : "PROCESS");
+#endif
+          switch (status) {
+            case TRAV_FINISH:
+              goto done_this_dir;
+            case TRAV_IGNORE:
+              goto next_path;
+            case TRAV_PROCESS:
+              if (stat(xpath, &sb2) >= 0) {
+                if (S_ISREG(sb2.st_mode)) {
+                  appended_any |= append_shallow(xpath, base_len, &sb2, list, methods, omit_globs);
+                } else if (S_ISDIR(sb2.st_mode)) {
+                  appended_any |= append_deep(xpath, base_len, &sb2, list, methods, omit_globs);
+                }
+              }
+              break;
           }
         }
+next_path:
+        (void) 0;
       }
+done_this_dir:
       closedir(d);
     }
     free(xpath);
   }
+  return appended_any;
 }
 /*}}}*/
 static void handle_wild(char *path, int base_len, char *last_comp, struct string_list *list,/*{{{*/
-                        void (*append)(char *, int, struct stat *, struct string_list *,
-                              int(*)(const char*, struct stat *), struct globber_array *),
-                        int (*filter)(const char *, struct stat *),
+                        int (*append)(char *, int, struct stat *, struct string_list *,
+                              const struct traverse_methods *, struct globber_array *),
+                        const struct traverse_methods *methods,
                         struct globber_array *omit_globs)
 {
   /* last_comp is the character within 'path' where the wildcard stuff starts. */
@@ -623,7 +651,7 @@ static void handle_wild(char *path, int base_len, char *last_comp, struct string
            * later anyway. */
           had_matches = 1;
           if (stat(xpath, &xsb) >= 0) {
-            (*append)(xpath, base_len, &xsb, list, filter, omit_globs);
+            (*append)(xpath, base_len, &xsb, list, methods, omit_globs);
           }
         }
       }
@@ -643,20 +671,20 @@ static void handle_wild(char *path, int base_len, char *last_comp, struct string
 }
 /*}}}*/
 static void handle_single(char *path, int base_len, struct string_list *list,/*{{{*/
-                          void (*append)(char *, int, struct stat *, struct string_list *,
-                                int(*)(const char*, struct stat *), struct globber_array *),
-                          int (*filter)(const char *, struct stat *),
+                          int (*append)(char *, int, struct stat *, struct string_list *,
+                                const struct traverse_methods *, struct globber_array *),
+                          const struct traverse_methods *methods,
                           struct globber_array *omit_globs)
 {
   struct stat sb;
   if (stat(path, &sb) >= 0) {
-    (*append)(path, base_len, &sb, list, filter, omit_globs);
+    (*append)(path, base_len, &sb, list, methods, omit_globs);
   } else {
     fprintf(stderr, "WARNING: Folder path %s does not exist\n", path);
   }
 }
 /*}}}*/
-int filter_is_file(const char *x, struct stat *sb)/*{{{*/
+static int filter_is_file(const char *x, const struct stat *sb)/*{{{*/
 {
   if (S_ISREG(sb->st_mode))
     return 1;
@@ -664,6 +692,17 @@ int filter_is_file(const char *x, struct stat *sb)/*{{{*/
     return 0;
 }
 /*}}}*/
+enum traverse_check scrutinize_mbox_entry(int parent_is_mbox, const char *de_name)/*{{{*/
+{
+  /* We have to keep looking at everything in this case. */
+  return TRAV_PROCESS;
+}
+/*}}}*/
+struct traverse_methods mbox_traverse_methods = {/*{{{*/
+  .filter = filter_is_file,
+  .scrutinize = scrutinize_mbox_entry
+};
+/*}}}*/
 static int is_wild(const char *x)/*{{{*/
 {
   const char *p;
@@ -680,7 +719,12 @@ static int is_wild(const char *x)/*{{{*/
   return 0;
 }
 /*}}}*/
-static void handle_one_path(const char *folder_base, const char *path, struct string_list *list, int (*filter)(const char *, struct stat *), struct globber_array *omit_globs)/*{{{*/
+/*{{{ handle_one_path() */
+static void handle_one_path(const char *folder_base,
+    const char *path,
+    struct string_list *list,
+    const struct traverse_methods *methods,
+    struct globber_array *omit_globs)
 {
   /* Valid syntaxen ([.]=optional):
    * [xxx/]foo : single path
@@ -720,21 +764,26 @@ static void handle_one_path(const char *folder_base, const char *path, struct st
   if ((len >= 4) && !strcmp(full_path + (len - 3), "...")) {
     full_path[len - 3] = '\0';
     if (is_wild(last_comp)) {
-      handle_wild(full_path, base_len, last_comp, list, append_deep, filter, omit_globs);
+      handle_wild(full_path, base_len, last_comp, list, append_deep, methods, omit_globs);
     } else {
-      handle_single(full_path, base_len, list, append_deep, filter, omit_globs);
+      handle_single(full_path, base_len, list, append_deep, methods, omit_globs);
     }
   } else {
     if (is_wild(last_comp)) {
-      handle_wild(full_path, base_len, last_comp, list, append_shallow, filter, omit_globs);
+      handle_wild(full_path, base_len, last_comp, list, append_shallow, methods, omit_globs);
     } else {
-      handle_single(full_path, base_len, list, append_shallow, filter, omit_globs);
+      handle_single(full_path, base_len, list, append_shallow, methods, omit_globs);
     }
   }
   free(full_path);
 }
 /*}}}*/
-void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, char ***paths_out, int *n_out, int (*filter)(const char *, struct stat *), struct globber_array *omit_globs)/*{{{*/
+/*{{{ glob_and_expand_paths() */
+void glob_and_expand_paths(const char *folder_base,
+    char **paths_in, int n_in,
+    char ***paths_out, int *n_out,
+    const struct traverse_methods *methods,
+    struct globber_array *omit_globs)
 {
   struct string_list list;
   int i;
@@ -744,7 +793,7 @@ void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, c
 
   for (i=0; i<n_in; i++) {
     char *path = paths_in[i];
-    handle_one_path(folder_base, path, &list, filter, omit_globs); 
+    handle_one_path(folder_base, path, &list, methods, omit_globs); 
   }
 
   string_list_to_array(&list, n_out, paths_out);
@@ -766,7 +815,7 @@ void build_mbox_lists(struct database *db, const char *folder_base, /*{{{*/
 
   if (mboxen_paths) {
     split_on_colons(mboxen_paths, &n_raw_paths, &raw_paths);
-    glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_file, omit_globs);
+    glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, &mbox_traverse_methods, omit_globs);
     extant_mboxen = new_array(struct extant_mbox, n_paths);
   } else {
     n_paths = 0;
@@ -822,8 +871,10 @@ void build_mbox_lists(struct database *db, const char *folder_base, /*{{{*/
         int len;
         create_ro_mapping(mb->path, &va, &len);
         if (va) {
-          rescan_mbox(mb, va, len);
-          munmap(va, (size_t) len);
+          rescan_mbox(mb, (char *) va, len);
+          free_ro_mapping(va, len);
+        } else if (!len) {
+          mb->n_old_msgs_valid = mb->n_msgs = 0;
         } else {
           /* Treat as dead mbox */
           deaden_mbox(mb);
@@ -880,7 +931,7 @@ int add_mbox_messages(struct database *db)/*{{{*/
       start = mb->start[j];
       len   = mb->len[j];
       msg_src = setup_msg_src(mb->path, start, len);
-      r8 = data_to_rfc822(msg_src, va + start, len);
+      r8 = data_to_rfc822(msg_src, (char *) va + start, len);
       if (r8) {
         if (verbose) {
           printf("Scanning %s[%d] at [%d,%d)\n", mb->path, j, (int)start, (int)(start + len));
@@ -895,7 +946,9 @@ int add_mbox_messages(struct database *db)/*{{{*/
       ++db->n_msgs;
       any_new = 1;
     }
-    if (va) munmap(va, (size_t) valen);
+    if (va) {
+    	free_ro_mapping(va, valen);
+    }
   }
   return any_new;
 }
