@@ -352,7 +352,7 @@ static void match_substring_in_paths(struct read_db *db, char *substring, int ma
   }
   for (i=0; i<db->n_msgs; i++) {
     char *token = NULL;
-    int mbix, msgix;
+    unsigned int mbix, msgix;
     switch (db->msg_type[i]) {
       case DB_MSG_FILE:
         token = db->data + db->path_offsets[i];
@@ -492,6 +492,7 @@ static void find_size_matches_in_table(struct read_db *db, char *size_expr, char
   int has_start, has_end, start_cond, end_cond;
   int i;
   
+  start = end = -1; /* avoid compiler warning about uninitialised variables. */
   parse_size_range(size_expr, &has_start, &start, &has_end, &end);
   if (has_start && has_end) {
     /* Allow user to put the endpoints in backwards */
@@ -542,7 +543,7 @@ static void find_date_matches_in_table(struct read_db *db, char *date_expr, char
 }
 /*}}}*/
 
-static char *mk_maildir_path(int token, char *output_dir)/*{{{*/
+static char *mk_maildir_path(int token, char *output_dir, int is_in_new)/*{{{*/
 {
   char *result; 
   char uniq_buf[48];
@@ -551,9 +552,12 @@ static char *mk_maildir_path(int token, char *output_dir)/*{{{*/
   len = strlen(output_dir) + 64; /* oversize */
   result = new_array(char, len);
   strcpy(result, output_dir);
-  strcat(result, "/cur/");
-  sprintf(uniq_buf, "123456789.%d.mairix:2,S", token);
+  strcat(result, is_in_new ? "/new/" : "/cur/");
+  sprintf(uniq_buf, "123456789.%d.mairix", token);
   strcat(result, uniq_buf);
+  if (!is_in_new) {
+    strcat(uniq_buf, ":2,S");
+  }
   return result;
 }
 /*}}}*/
@@ -572,6 +576,23 @@ static char *mk_mh_path(int token, char *output_dir)/*{{{*/
   return result;
 }
 /*}}}*/
+static int looks_like_maildir_new_p(const char *p)/*{{{*/
+{
+  const char *s1, *s2;
+  s2 = p;
+  while (*s2) s2++;
+  while ((s2 > p) && (*s2 != '/')) s2--;
+  if (s2 <= p) return 0;
+  s1 = s2 - 1;
+  while ((s1 > p) && (*s1 != '/')) s1--;
+  if (s1 <= p) return 0;
+  if (!strncmp(s1, "/new/", 5)) {
+    return 1;
+  } else {
+    return 0;
+  }
+}
+/*}}}*/
 static void create_symlink(char *link_target, char *new_link)/*{{{*/
 {
   if (symlink(link_target, new_link) < 0) {
@@ -589,8 +610,9 @@ static void append_file_to_mbox(const char *path, FILE *out)/*{{{*/
   create_ro_mapping(path, &data, &len);
   if (data) {
     fprintf(out, "From mairix@mairix Mon Jan  1 12:34:56 1970\n");
+    fprintf(out, "X-source-folder: %s\n", path);
     fwrite (data, sizeof(unsigned char), len, out);
-    munmap(data, len);
+    free_ro_mapping(data, len);
   }
   return;
 }
@@ -619,7 +641,7 @@ static void get_validated_mbox_msg(struct read_db *db, int msg_index,/*{{{*/
 
   start = *mbox_data + db->mtime_table[msg_index];
   *msg_len = db->size_table[msg_index];
-  compute_checksum(start, *msg_len, &csum);
+  compute_checksum((char *)start, *msg_len, &csum);
   if (!memcmp((db->data + db->mbox_checksum_table[mbi] + (msgi * sizeof(checksum_t))), &csum, sizeof(checksum_t))) {
     *msg_data = start;
   } else {
@@ -637,11 +659,15 @@ static void append_mboxmsg_to_mbox(struct read_db *db, int msg_index, FILE *out)
 
   get_validated_mbox_msg(db, msg_index, &mbox_index, &mbox_start, &mbox_len, &msg_start, &msg_len);
   if (msg_start) {
+    /* Artificial from line, we don't have the envelope sender so this is
+       going to be artificial anyway. */
     fprintf(out, "From mairix@mairix Mon Jan  1 12:34:56 1970\n");
+    fprintf(out, "X-source-folder: %s\n",
+            db->data + db->mbox_paths_table[mbox_index]);
     fwrite(msg_start, sizeof(unsigned char), msg_len, out);
   }
   if (mbox_start) {
-    munmap(mbox_start, mbox_len);
+    free_ro_mapping(mbox_start, mbox_len);
   }
 }
 /*}}}*/
@@ -658,15 +684,15 @@ static void try_copy_to_path(struct read_db *db, int msg_index, char *target_pat
   if (start) {
     out = fopen(target_path, "wb");
     if (out) {
-      /* Artificial from line, we don't have the envelope sender so this is
-         going to be artificial anyway. */
+      fprintf(out, "X-source-folder: %s\n",
+              db->data + db->mbox_paths_table[mbi]);
       fwrite(start, sizeof(char), msg_len, out);
       fclose(out);
     }
   }
 
   if (data) {
-    munmap(data, mbox_len);
+    free_ro_mapping(data, mbox_len);
   }
   return;
 }
@@ -919,15 +945,20 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
           switch (db->msg_type[i]) {
             case DB_MSG_FILE:
               {
-                char *target_path = mk_maildir_path(i, output_path);
-                create_symlink(db->data + db->path_offsets[i], target_path);
+                char *target_path;
+                char *message_path;
+                int is_in_new;
+                message_path = db->data + db->path_offsets[i];
+                is_in_new = looks_like_maildir_new_p(message_path);
+                target_path = mk_maildir_path(i, output_path, is_in_new);
+                create_symlink(message_path, target_path);
                 free(target_path);
                 ++n_hits;
               }
               break;
             case DB_MSG_MBOX:
               {
-                char *target_path = mk_maildir_path(i, output_path);
+                char *target_path = mk_maildir_path(i, output_path, 0);
                 try_copy_to_path(db, i, target_path);
                 free(target_path);
                 ++n_hits;
@@ -1013,7 +1044,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
               break;
             case DB_MSG_MBOX:
               {
-                int mbix, msgix;
+                unsigned int mbix, msgix;
                 int start, len, after_end;
                 start = db->mtime_table[i];
                 len   = db->size_table[i];
@@ -1169,7 +1200,7 @@ static void clear_mh_folder(char *path)/*{{{*/
   d = opendir(path);
   if (d) {
     while ((de = readdir(d))) {
-      if (is_integer_string(de->d_name)) {
+      if (valid_mh_filename_p(de->d_name)) {
         strcpy(fpath, path);
         strcat(fpath, "/");
         strcat(fpath, de->d_name);
